@@ -34,6 +34,14 @@ const PREDEFINED_SITELIST = [
     'https://*.wordpress.com/log-in/'
 ];
 
+const IMPROVED_DETECTION_PREDEFINED_SITELIST = [
+    'https://auth.max.com/',
+    'https://login.qt.io/login',
+    'https://secure.chase.com/*',
+    'https://www.reddit.com/',
+    'https://www.icloud.com/'
+];
+
 const googleUrl = 'https://accounts.google.com';
 
 const kpxcSites = {};
@@ -83,7 +91,8 @@ kpxcSites.exceptionFound = function(identifier, field) {
               || document.location.origin.startsWith('https://mail.protonmail.com')
               && identifier === 'mailboxPassword') {
         return true;
-    } else if (document.location.origin === 'https://www.patreon.com' && field?.name === 'current-password') {
+    } else if (document.location.origin === 'https://www.patreon.com' &&
+               (field?.name === 'current-password' || field?.innerHTML?.includes('current-password'))) {
         return true;
     } else if (document.location.origin === 'https://wordpress.com' && identifier?.value === 'login__form-password') {
         return true;
@@ -98,7 +107,7 @@ kpxcSites.exceptionFound = function(identifier, field) {
  * @returns {boolean}      True if an Element has a match with the needed indentfifiers and document location
  */
 kpxcSites.totpExceptionFound = function(field) {
-    if (!field || field.nodeName !== 'INPUT') {
+    if (!field || !matchesWithNodeName(field, 'INPUT')) {
         return false;
     }
 
@@ -117,7 +126,7 @@ kpxcSites.totpExceptionFound = function(field) {
  * @returns {boolean}       True if an Element has a match with the needed indentfifiers and document location
  */
 kpxcSites.segmentedTotpExceptionFound = function(form) {
-    if (!form || form.nodeName !== 'FORM') {
+    if (!form || !matchesWithNodeName(form, 'FORM')) {
         return false;
     }
 
@@ -143,11 +152,11 @@ kpxcSites.expectedTOTPMaxLength = function() {
 
 /**
  * Handles a few exceptions for certain sites where form submit button is not regognized properly.
- * @param {object} form     Form element
+ * @param {object} form     Form element (optional)
  * @returns {object}        Button element
  */
 kpxcSites.formSubmitButtonExceptionFound = function(form) {
-    if (form.action.startsWith(googleUrl)) {
+    if (form?.action?.startsWith(googleUrl)) {
         const findDiv = $('#identifierNext, #passwordNext');
         if (!findDiv) {
             return undefined;
@@ -156,14 +165,14 @@ kpxcSites.formSubmitButtonExceptionFound = function(form) {
         const buttons = findDiv.getElementsByTagName('button');
         kpxcSites.savedForm = form;
         return buttons.length > 0 ? buttons[0] : undefined;
-    } else if (form.action.startsWith('https://www.ebay.')) {
+    } else if (form?.action?.startsWith('https://www.ebay.')) {
         // For eBay we must return the first button.
         for (const i of form.elements) {
             if (i.type === 'button') {
                 return i;
             }
         }
-    } else if (form.action.includes('signin.aws.amazon.com')) {
+    } else if (form?.action?.includes('signin.aws.amazon.com')) {
         // For Amazon AWS the button is outside the form.
         const button = $('#signin_button');
         if (button) {
@@ -176,14 +185,17 @@ kpxcSites.formSubmitButtonExceptionFound = function(form) {
             'odc.officeapps.live.com',
             'login.microsoftonline.com',
             'login.microsoftonline.us',
-        ].some(u => form.action.includes(u))) {
+        ].some(u => form?.action?.includes(u))) {
         const buttons = Array.from(form.querySelectorAll(kpxcForm.formButtonQuery));
         if (buttons?.length > 1) {
             return buttons[1];
         }
-    } else if (form.action.startsWith('https://barmerid.id.bconnect.barmer.de')) {
+    } else if (form?.action?.startsWith('https://barmerid.id.bconnect.barmer.de')) {
         const loginButton = $('#btn-login');
         return loginButton?.shadowRoot?.children?.[0];
+    } else if (!form && document.location.href.includes('reddit.com/settings')) {
+        // Reddit change password popup
+        return $('.button[slot=primary-button]');
     }
 
     return undefined;
@@ -204,14 +216,17 @@ kpxcSites.popupExceptionFound = function(combinations) {
 
 /**
  * Handles a few exceptions for certain sites where Username Icon is not placed properly.
- * @param {number} left     Absolute left position of the icon
- * @param {number} top      Absolute top position of the icon
- * @param {number} iconSize Size of the icon
- * @returns {array}         New left and top values as an Array
+ * @param {number} left         Absolute left position of the icon
+ * @param {number} top          Absolute top position of the icon
+ * @param {number} iconSize     Size of the icon
+ * @param {string} inputType    Input field type
+ * @returns {array}             New left and top values as an Array
  */
-kpxcSites.iconOffset = function(left, top, iconSize) {
+kpxcSites.iconOffset = function(left, top, iconSize, inputType) {
     if (document.location.hostname.includes('idmsa.apple.com')) {
         return [ left - (iconSize + 10), top + 3 ];
+    } else if (document.location.origin === 'https://secure.royalbank.com' && inputType === 'password') {
+        return [ left - (iconSize + 10), top ];
     }
 
     return undefined;
