@@ -1,8 +1,8 @@
 'use strict';
 
 const PASSKEYS_NO_LOGINS_FOUND = 15;
-const PASSKEYS_WAIT_FOR_LIFETIMER = 21;
-const PASSKEYS_CREDENTIAL_IS_EXCLUDED = 30;
+const PASSKEYS_CREDENTIAL_IS_EXCLUDED = 21;
+const PASSKEYS_WAIT_FOR_LIFETIMER = 30;
 
 // Contains already called method names
 const _called = {};
@@ -327,15 +327,20 @@ kpxc.initCombinations = async function(inputs = []) {
         const field = c.username || c.password;
         if (field && c.form) {
             // Initialize form-submit for remembering credentials
-            kpxcForm.init(c.form, c);
+            kpxcForm.initForm(c.form, c);
         }
 
         // Don't allow duplicates
-        if (!kpxc.combinations.some(f => f.username === c.username && f.password === c.password && f.totp === c.totp && f.form === c.form)) {
+        if (!kpxc.combinations.some(f =>
+            f.username === c.username && f.password === c.password && f.totp === c.totp && f.form === c.form
+        )) {
             kpxc.combinations.push(c);
         }
     }
 
+    // Identify a custom password change form
+    kpxcForm.initCustomForm(combinations);
+
     // Update the fields in Custom Login Fields banner if it's open
     if (kpxcCustomLoginFieldsBanner.created) {
         kpxcCustomLoginFieldsBanner.updateFieldSelections();
@@ -600,7 +605,7 @@ kpxc.rememberCredentialsFromContextMenu = async function() {
     }
 
     const el = document.activeElement;
-    if (el.nodeName !== 'INPUT') {
+    if (!matchesWithNodeName(el, 'INPUT')) {
         return;
     }
 
@@ -698,7 +703,7 @@ kpxc.setPasswordFilled = async function(state) {
     await sendMessage('password_set_filled', state);
 };
 
-// Special handling for settings value to select element
+// Special handling for setting value to select and checkbox elements
 kpxc.setValue = function(field, value, forced = false) {
     if (field.matches('select')) {
         value = value.toLowerCase().trim();
@@ -712,6 +717,8 @@ kpxc.setValue = function(field, value, forced = false) {
         }
 
         return;
+    } else if (field.getLowerCaseAttribute('type') === 'checkbox' && value?.toLowerCase() === 'true') {
+        field.checked = true;
     }
 
     kpxc.setValueWithChange(field, value, forced);
@@ -747,7 +754,7 @@ kpxc.showGroupNameInAutocomplete = function() {
         || (kpxc.settings.showGroupNameInAutocomplete && kpxc.getUniqueGroupCount(kpxc.credentials) > 1);
 };
 
-// Returns true if site is ignored
+// Returns true if site is ignored, and checks for predefined sites
 kpxc.siteIgnored = async function(condition) {
     if (kpxc.settings.sitePreferences) {
         let currentLocation;
@@ -760,7 +767,6 @@ kpxc.siteIgnored = async function(condition) {
             currentLocation = window.self.location.href;
         }
 
-        // Refresh current settings for the site
         const currentSetting = condition || IGNORE_FULL;
         for (const site of kpxc.settings.sitePreferences) {
             if (siteMatch(site.url, currentLocation) || site.url === currentLocation) {
@@ -768,6 +774,7 @@ kpxc.siteIgnored = async function(condition) {
                     return true;
                 }
 
+                // Refresh current settings for the site
                 kpxc.singleInputEnabledForPage = site.usernameOnly;
                 kpxc.improvedFieldDetectionEnabledForPage = site.improvedFieldDetection;
                 await sendMessage('page_set_allow_iframes', [ site.allowIframes, currentLocation ]);
@@ -776,11 +783,7 @@ kpxc.siteIgnored = async function(condition) {
 
         // Check for predefined sites
         if (kpxc.settings.usePredefinedSites) {
-            for (const url of PREDEFINED_SITELIST) {
-                if (siteMatch(url, currentLocation) || url === currentLocation) {
-                    kpxc.singleInputEnabledForPage = true;
-                }
-            }
+            kpxc.usePredefinedSites(currentLocation);
         }
     }
 
@@ -843,9 +846,26 @@ kpxc.updateTOTPList = async function() {
     return [];
 };
 
+// Enable certain settings based on predefined sites list
+kpxc.usePredefinedSites = function(currentLocation) {
+    // Single input field
+    for (const url of PREDEFINED_SITELIST) {
+        if (siteMatch(url, currentLocation) || url === currentLocation) {
+            kpxc.singleInputEnabledForPage = true;
+        }
+    }
+
+    // Improved input field detection
+    for (const url of IMPROVED_DETECTION_PREDEFINED_SITELIST) {
+        if (siteMatch(url, currentLocation) || url === currentLocation) {
+            kpxc.improvedFieldDetectionEnabledForPage = true;
+        }
+    }
+};
+
 // Apply a script to the page for intercepting Passkeys (WebAuthn) requests
 kpxc.enablePasskeys = function() {
-    if (document?.documentElement?.ownerDocument?.contentType === 'text/xml') {
+    if (document?.documentElement?.ownerDocument?.contentType !== 'text/html') {
         return;
     }
 
@@ -925,6 +945,12 @@ kpxc.enablePasskeys = function() {
  */
 const initContentScript = async function() {
     try {
+        if (document?.documentElement?.ownerDocument?.contentType !== 'text/html'
+            && document?.documentElement?.ownerDocument?.contentType !== 'application/xhtml+xml'
+        ) {
+            return;
+        }
+
         const settings = await sendMessage('load_settings');
         if (!settings) {
             logError('Error: Cannot load extension settings');
