@@ -31,9 +31,10 @@ const createAttestationResponse = function(publicKey) {
         attestationObject: kpxcBase64ToArrayBuffer(publicKey.response.attestationObject),
         clientDataJSON: kpxcBase64ToArrayBuffer(publicKey.response.clientDataJSON),
         getAuthenticatorData: () => kpxcBase64ToArrayBuffer(publicKey.response?.authenticatorData),
+        getPublicKey: () => null,
+        getPublicKeyAlgorithm: () => publicKey.response?.publicKeyAlgorithm,
         getTransports: () => [ 'internal' ]
     };
-
     return Object.setPrototypeOf(response, AuthenticatorAttestationResponse.prototype);
 };
 
@@ -187,6 +188,9 @@ const throwError = function(errorCode, errorMessage) {
             }
 
             return createPublicKeyCredential(response.publicKey);
+        },
+        async store(credential) {
+            return originalCredentials.store(credential);
         }
     };
 
